@@ -33,12 +33,12 @@ static void parseCompactIfopt(ScopedXmlStreamReader &reader, Location &loc)
     if (reader.name() == QLatin1String("pgid")) {
         const auto id = reader.readElementText();
         if (IfoptUtil::isValid(id)) {
-            loc.setIdentifier(QStringLiteral("ifopt"), id);
+            loc.setIdentifier(IfoptUtil::identifierType(), id);
         }
     } else if (reader.name() == QLatin1String("gid")) {
         const auto id = reader.readElementText();
-        if (IfoptUtil::isValid(id) && loc.identifier(QStringLiteral("ifopt")).isEmpty()) {
-            loc.setIdentifier(QStringLiteral("ifopt"), id);
+        if (IfoptUtil::isValid(id) && loc.identifier(IfoptUtil::identifierType()).isEmpty()) {
+            loc.setIdentifier(IfoptUtil::identifierType(), id);
         }
     }
 }
@@ -79,7 +79,7 @@ Location EfaCompactParser::parseCompactSf(ScopedXmlStreamReader &&reader) const
     return loc;
 }
 
-std::vector<Location> EfaCompactParser::parseStopFinderResponse(const QByteArray &data) const
+std::vector<Location> EfaCompactParser::parseStopFinderResponse(const QByteArray &data)
 {
     std::vector<Location> res;
     QXmlStreamReader xsr(data);
@@ -190,13 +190,17 @@ Stopover EfaCompactParser::parseCompactDp(ScopedXmlStreamReader &&reader) const
     return dep;
 }
 
-std::vector<Stopover> EfaCompactParser::parseDmResponse(const QByteArray &data) const
+std::vector<Stopover> EfaCompactParser::parseDmResponse(const QByteArray &data)
 {
     std::vector<Stopover> res;
     QXmlStreamReader xsr(data);
     ScopedXmlStreamReader reader(xsr);
     while (reader.readNextElement()) {
-        if (reader.name() == QLatin1String("dp")) {
+        if (reader.name() == QLatin1String("pas")) {
+            const auto pas = parseKeyValueList(reader.subReader(), QLatin1String("pa"), QLatin1String("n"), QLatin1String("v"));
+            m_requestContext.sessionId = pas.value(QStringLiteral("sessionID"));
+            m_requestContext.requestId = pas.value(QStringLiteral("requestID"));
+        } else if (reader.name() == QLatin1String("dp")) {
             res.push_back(parseCompactDp(reader.subReader()));
         }
     }
@@ -346,9 +350,9 @@ JourneySection EfaCompactParser::parseTripSection(ScopedXmlStreamReader &&reader
                     loc.setCoordinate(coord[1].toFloat(), coord[0].toFloat());
 
                     if (IfoptUtil::isValid(stopParams[13])) {
-                        loc.setIdentifier(QStringLiteral("ifopt"), stopParams[13]);
+                        loc.setIdentifier(IfoptUtil::identifierType(), stopParams[13]);
                     } else if (IfoptUtil::isValid(stopParams[12])) {
-                        loc.setIdentifier(QStringLiteral("ifopt"), stopParams[12]);
+                        loc.setIdentifier(IfoptUtil::identifierType(), stopParams[12]);
                     }
 
                     const auto dt = QDateTime::fromString(stopParams[2] + stopParams[3], QStringLiteral("yyyyMMddhhmm"));
@@ -379,7 +383,7 @@ JourneySection EfaCompactParser::parseTripSection(ScopedXmlStreamReader &&reader
             }
             section.setIntermediateStops(std::move(stops));
         } else if (reader.name() == QLatin1String("pt")) {
-            section.setPath(parsePathCoordinatesElement(reader));
+            section.setPath(polygonToPath(parsePathCoordinatesElement(reader)));
         }
         // TODO interchange tag - should we turn this into transfer sections?
     }
@@ -401,14 +405,18 @@ Journey EfaCompactParser::parseCompactTp(ScopedXmlStreamReader &&reader) const
     return jny;
 }
 
-std::vector<Journey> EfaCompactParser::parseTripResponse(const QByteArray &data) const
+std::vector<Journey> EfaCompactParser::parseTripResponse(const QByteArray &data)
 {
     //qDebug().noquote() << data;
     std::vector<Journey> res;
     QXmlStreamReader xsr(data);
     ScopedXmlStreamReader reader(xsr);
     while (reader.readNextElement()) {
-        if (reader.name() == QLatin1String("tp")) {
+        if (reader.name() == QLatin1String("pas")) {
+            const auto pas = parseKeyValueList(reader.subReader(), QLatin1String("pa"), QLatin1String("n"), QLatin1String("v"));
+            m_requestContext.sessionId = pas.value(QStringLiteral("sessionID"));
+            m_requestContext.requestId = pas.value(QStringLiteral("requestID"));
+        } else if (reader.name() == QLatin1String("tp")) {
             res.push_back(parseCompactTp(reader.subReader()));
         }
     }
