@@ -45,7 +45,7 @@ void EfaXmlParser::parseLocationCommon(Location &loc, const ScopedXmlStreamReade
     for (const auto &attr : {QLatin1String("pointGid"), QLatin1String("areaGid"), QLatin1String("gid")}) {
         const auto id = reader.attributes().value(attr);
         if (IfoptUtil::isValid(id)) {
-            loc.setIdentifier(QStringLiteral("ifopt"), id.toString());
+            loc.setIdentifier(IfoptUtil::identifierType(), id.toString());
             break;
         }
     }
@@ -77,7 +77,7 @@ Location EfaXmlParser::parseOdvNameElem(ScopedXmlStreamReader &reader) const
     return loc;
 }
 
-std::vector<Location> EfaXmlParser::parseStopFinderResponse(const QByteArray &data) const
+std::vector<Location> EfaXmlParser::parseStopFinderResponse(const QByteArray &data)
 {
     std::vector<Location> res;
     QXmlStreamReader xsr(data);
@@ -176,13 +176,17 @@ Stopover EfaXmlParser::parseDmDeparture(ScopedXmlStreamReader &&reader) const
     return dep;
 }
 
-std::vector<Stopover> EfaXmlParser::parseDmResponse(const QByteArray &data) const
+std::vector<Stopover> EfaXmlParser::parseDmResponse(const QByteArray &data)
 {
     std::vector<Stopover> res;
     QXmlStreamReader xsr(data);
     ScopedXmlStreamReader reader(xsr);
     while (reader.readNextElement()) {
-        if (reader.name() == QLatin1String("itdOdvAssignedStop") && reader.attributes().hasAttribute(QLatin1String("stopID"))) {
+        if (reader.name() == QLatin1String("itdRequest")) {
+            m_requestContext.sessionId = reader.attributes().value(QLatin1String("sessionID")).toString();
+        } else if (reader.name() == QLatin1String("itdDepartureMonitorRequest")) {
+            m_requestContext.requestId = reader.attributes().value(QLatin1String("requestID")).toString();
+        } else if (reader.name() == QLatin1String("itdOdvAssignedStop") && reader.attributes().hasAttribute(QLatin1String("stopID"))) {
             const auto loc = parseItdOdvAssignedStop(reader);
             m_locations[loc.identifier(m_locationIdentifierType)] = loc;
         } else if (reader.name() == QLatin1String("itdDeparture")) {
@@ -295,13 +299,16 @@ struct {
     { 100, JourneySection::Walking },
 };
 
-JourneySection EfaXmlParser::parseTripPartialRoute(ScopedXmlStreamReader &&reader) const
+std::vector<JourneySection> EfaXmlParser::parseTripPartialRoute(ScopedXmlStreamReader &&reader) const
 {
+    std::vector<JourneySection> result;
     JourneySection section;
     if (reader.attributes().value(QLatin1String("type")) == QLatin1String("IT")) {
         section.setMode(JourneySection::Walking);
     }
 
+    QPolygonF sectionPoly, transferPoly;
+    std::vector<PathDescription> pathDesc;
     while (reader.readNextSibling()) {
         if (reader.name() == QLatin1String("itdPoint")) {
             const auto type = reader.attributes().value(QLatin1String("usage"));
@@ -311,8 +318,6 @@ JourneySection EfaXmlParser::parseTripPartialRoute(ScopedXmlStreamReader &&reade
                 parseTripArrival(reader.subReader(), section);
             }
         } else if (reader.name() == QLatin1String("itdMeansOfTransport")) {
-            Line line;
-            line.setName(reader.attributes().value(QLatin1String("shortname")).toString());
             const auto type = reader.attributes().value(QLatin1String("type")).toInt();
             for (const auto &m : journey_section_types) {
                 if (m.type == type) {
@@ -320,19 +325,26 @@ JourneySection EfaXmlParser::parseTripPartialRoute(ScopedXmlStreamReader &&reade
                     break;
                 }
             }
-            const auto prodName = reader.attributes().value(QLatin1String("productName"));
-            if (prodName == QLatin1String("Fussweg")) {
-                section.setMode(JourneySection::Walking);
-            } else {
-                line.setModeString(prodName.toString());
-            }
-            line.setMode(motTypeToLineMode(reader.attributes().value(QLatin1String("motType")).toInt()));
-            Route route;
-            route.setDirection(reader.attributes().value(QLatin1String("destination")).toString());
-            route.setLine(line);
-            section.setRoute(route);
-            if (section.mode() == JourneySection::Invalid) {
+
+            if (type < 90) {
+                Line line;
+                line.setName(reader.attributes().value(QLatin1String("shortname")).toString());
+                const auto prodName = reader.attributes().value(QLatin1String("productName"));
+                if (prodName == QLatin1String("Fussweg")) {
+                    section.setMode(JourneySection::Walking);
+                } else {
+                    line.setModeString(prodName.toString());
+                }
+                line.setMode(motTypeToLineMode(reader.attributes().value(QLatin1String("motType")).toInt()));
+                Route route;
+                route.setDirection(reader.attributes().value(QLatin1String("destination")).toString());
+                route.setLine(line);
+                section.setRoute(route);
                 section.setMode(JourneySection::PublicTransport);
+            } else if (type > 100) {
+                const auto itMode = motTypeToIndividualTransportMode(type);
+                section.setIndividualTransport(itMode);
+                section.setMode(JourneySection::IndividualTransport);
             }
         } else if (reader.name() == QLatin1String("infoLink")) {
             section.addNotes(parseInfoLink(reader.subReader()));
@@ -346,19 +358,57 @@ JourneySection EfaXmlParser::parseTripPartialRoute(ScopedXmlStreamReader &&reade
         } else if (reader.name() == QLatin1String("itdStopSeq")) {
             section.setIntermediateStops(parsePartialTripStopSequence(reader.subReader()));
         } else if (reader.name() == QLatin1String("itdPathCoordinates")) {
-            section.setPath(parsePathCoordinates(reader.subReader()));
-       } else if (reader.name() == QLatin1String("itdITPathDescription") && !section.path().isEmpty()) {
+            sectionPoly = parsePathCoordinates(reader.subReader());
+        } else if (reader.name() == QLatin1String("itdInterchangePathCoordinates")) {
+            auto subreader = reader.subReader();
+            while (subreader.readNextSibling()) {
+                if (subreader.name() == QLatin1String("itdPathCoordinates")) {
+                    transferPoly = parsePathCoordinates(subreader.subReader());
+                }
+            }
+        } else if (reader.name() == QLatin1String("itdITPathDescription") && !sectionPoly.isEmpty()) {
             auto subreader = reader.subReader();
             while (subreader.readNextSibling()) {
                 if (subreader.name() == QLatin1String("itdITPathDescriptionList")) {
-                    const auto fullPath = section.path();
-                    section.setPath(parsePathDescriptionList(subreader.subReader(), fullPath.sections()[0].path()));
+                    pathDesc = parsePathDescriptionList(subreader.subReader());
                 }
             }
         }
     }
 
-    return section;
+    if (!pathDesc.empty()) {
+        resolvePathDescription(pathDesc);
+        if (!transferPoly.empty() && section.mode() == JourneySection::PublicTransport) {
+            // path description is actually for a subsequent transfer section
+            const auto path = assemblePath(pathDesc, transferPoly);
+            JourneySection transferSection;
+            transferSection.setMode(JourneySection::Transfer);
+            transferSection.setScheduledDepartureTime(section.scheduledArrivalTime());
+            const auto travelTime = std::accumulate(pathDesc.begin(), pathDesc.end(), 0, [](auto accu, const auto &desc) {
+                return accu + desc.travelTime;
+            });
+            transferSection.setScheduledArrivalTime(section.scheduledArrivalTime().addSecs(travelTime));
+            Location from;
+            from.setLatitude(path.startPoint().y());
+            from.setLongitude(path.startPoint().x());
+            transferSection.setFrom(std::move(from));
+            Location to;
+            to.setLatitude(path.endPoint().y());
+            to.setLongitude(path.endPoint().x());
+            transferSection.setTo(std::move(to));
+            transferSection.setPath(std::move(path));
+            result.push_back(std::move(transferSection));
+        } else if (section.mode() == JourneySection::Walking) {
+            // path description is for this section
+            section.setPath(assemblePath(pathDesc, sectionPoly));
+        }
+    }
+    if (!sectionPoly.isEmpty() && section.path().isEmpty()) {
+        section.setPath(polygonToPath(sectionPoly));
+    }
+
+    result.insert(result.begin(), std::move(section));
+    return result;
 }
 
 Journey EfaXmlParser::parseTripRoute(ScopedXmlStreamReader &&reader) const
@@ -368,7 +418,8 @@ Journey EfaXmlParser::parseTripRoute(ScopedXmlStreamReader &&reader) const
 
     while (reader.readNextElement()) {
         if (reader.name() == QLatin1String("itdPartialRoute")) {
-            sections.push_back(parseTripPartialRoute(reader.subReader()));
+            auto sec = parseTripPartialRoute(reader.subReader());
+            std::move(sec.begin(), sec.end(), std::back_inserter(sections));
         }
     }
 
@@ -376,14 +427,18 @@ Journey EfaXmlParser::parseTripRoute(ScopedXmlStreamReader &&reader) const
     return journey;
 }
 
-std::vector<Journey> EfaXmlParser::parseTripResponse(const QByteArray &data) const
+std::vector<Journey> EfaXmlParser::parseTripResponse(const QByteArray &data)
 {
     //qDebug().noquote() << data;
     std::vector<Journey> res;
     QXmlStreamReader xsr(data);
     ScopedXmlStreamReader reader(xsr);
     while (reader.readNextElement()) {
-        if (reader.name() == QLatin1String("itdRoute")) {
+        if (reader.name() == QLatin1String("itdRequest")) {
+            m_requestContext.sessionId = reader.attributes().value(QLatin1String("sessionID")).toString();
+        } else if (reader.name() == QLatin1String("itdTripRequest")) {
+            m_requestContext.requestId = reader.attributes().value(QLatin1String("requestID")).toString();
+        } else if (reader.name() == QLatin1String("itdRoute")) {
             res.push_back(parseTripRoute(reader.subReader()));
         }
     }
@@ -403,50 +458,101 @@ QStringList EfaXmlParser::parseInfoLink(ScopedXmlStreamReader &&reader) const
     return l;
 }
 
-Path EfaXmlParser::parsePathCoordinates(ScopedXmlStreamReader &&reader) const
+QPolygonF EfaXmlParser::parsePathCoordinates(ScopedXmlStreamReader &&reader) const
 {
-    Path path;
+    QPolygonF poly;
     while (reader.readNextSibling()) {
         if (reader.name() == QLatin1String("itdCoordinateString")) {
-            path = parsePathCoordinatesElement(reader);
+            poly = parsePathCoordinatesElement(reader);
         }
     }
-    return path;
+    return poly;
 }
 
-Path EfaXmlParser::parsePathDescriptionList(ScopedXmlStreamReader &&reader, const QPolygonF &poly) const
+std::vector<EfaXmlParser::PathDescription> EfaXmlParser::parsePathDescriptionList(ScopedXmlStreamReader &&reader) const
+{
+    std::vector<PathDescription> descs;
+    while (reader.readNextSibling()) {
+        if (reader.name() == QLatin1String("itdITPathDescriptionElem")) {
+            PathDescription desc;
+            auto elemReader = reader.subReader();
+            while (elemReader.readNextSibling()) {
+                if (elemReader.name() == QLatin1String("itdCoord")) {
+                    desc.point.setX(elemReader.attributes().value(QLatin1String("x")).toDouble());
+                    desc.point.setY(elemReader.attributes().value(QLatin1String("y")).toDouble());
+                } else if (elemReader.name() == QLatin1String("fromPathCoordIdx")) {
+                    desc.fromIndex = elemReader.readElementText().toInt();
+                } else if (elemReader.name() == QLatin1String("toPathCoordIdx")) {
+                    desc.toIndex = elemReader.readElementText().toInt();
+                } else if (elemReader.name() == QLatin1String("streetname")) {
+                    desc.description = elemReader.readElementText();
+                } else if (elemReader.name() == QLatin1String("traveltime")) {
+                    desc.travelTime = elemReader.readElementText().toInt();
+                } else if (elemReader.name() == QLatin1String("niveau")) {
+                    // seems to match OSM floor level, but is always "0" for elevators/escalators
+                    desc.niveau = elemReader.readElementText().toInt();
+                } else if (elemReader.name() == QLatin1String("genAttrList")) {
+                    const auto attrs = parseGenericAttributeList(elemReader.subReader());
+                    const auto indoorType = attrs.value(QStringLiteral("INDOOR_TYPE"));
+                    if (indoorType == QLatin1String("STAIRS")) {
+                        desc.maneuver = PathSection::Stairs;
+                    } else if (indoorType == QLatin1String("LIFT")) {
+                        desc.maneuver = PathSection::Elevator;
+                    } else if (indoorType == QLatin1String("ESCALATOR")) {
+                        desc.maneuver = PathSection::Escalator;
+                    }
+                }
+                // NOTE: skyDirection seems flipped by 180°, ie. pointing to the start point, should we ever need that
+                // turnDirection, turningManoeuvre, from/toPathLink??
+            }
+            descs.push_back(std::move(desc));
+        }
+    }
+    return descs;
+}
+
+void EfaXmlParser::resolvePathDescription(std::vector<PathDescription> &descs) const
+{
+    if (descs.size() < 3) {
+        return;
+    }
+
+    for (auto it = std::next(descs.begin()); it != std::prev(descs.end()); ++it) {
+        if ((*it).maneuver != PathSection::Stairs && (*it).maneuver != PathSection::Elevator && (*it).maneuver != PathSection::Escalator) {
+            continue;
+        }
+        const auto niveauBefore = (*std::prev(it)).niveau;
+        const auto niveauAfter = (*std::next(it)).niveau;
+        (*it).niveauDelta = niveauAfter - niveauBefore;
+    }
+}
+
+Path EfaXmlParser::assemblePath(const std::vector<PathDescription> &descs, const QPolygonF &poly) const
 {
     Path path;
     std::vector<PathSection> sections;
 
-    while (reader.readNextSibling()) {
-        if (reader.name() == QLatin1String("itdITPathDescriptionElem")) {
-            PathSection section;
-            int fromIdx = -1; int toIdx = -1;
-            auto elemReader = reader.subReader();
-            while (elemReader.readNextSibling()) {
-                if (elemReader.name() == QLatin1String("fromPathCoordIdx")) {
-                    fromIdx = elemReader.readElementText().toInt();
-                } else if (elemReader.name() == QLatin1String("toPathCoordIdx")) {
-                    toIdx = elemReader.readElementText().toInt();
-                } else if (elemReader.name() == QLatin1String("streetname")) {
-                    section.setDescription(elemReader.readElementText());
-                }
-                // turnDirection, turningManoeuvre, from/toPathLink??, skyDirection, traveltime, distance, niveau, genAttrList
-            }
-
-            if (fromIdx < 0 || toIdx < 0 || fromIdx >= poly.size() || toIdx >= poly.size() || toIdx < fromIdx) {
-                qWarning() << "weird polygon indexes?" << fromIdx << toIdx << poly.size();
-                continue;
-            }
-            QPolygonF subPoly;
-            subPoly.reserve(toIdx - fromIdx + 1);
-            std::copy(poly.begin() + fromIdx, poly.begin() + toIdx, std::back_inserter(subPoly));
-            section.setPath(subPoly);
-            sections.push_back(std::move(section));
+    for (const auto &desc : descs) {
+        if (desc.fromIndex < 0 || desc.toIndex < 0 || desc.fromIndex >= poly.size() || desc.toIndex >= poly.size() || desc.toIndex < desc.fromIndex) {
+            qCWarning(Log) << "weird polygon indexes?" << desc.fromIndex << desc.toIndex << poly.size();
+            continue;
         }
+        PathSection section;
+        QPolygonF subPoly;
+        subPoly.reserve(desc.toIndex - desc.fromIndex + 1);
+        std::copy(poly.begin() + desc.fromIndex, poly.begin() + desc.toIndex + 1, std::back_inserter(subPoly));
+        section.setPath(subPoly);
+        section.setDescription(desc.description);
+        section.setManeuver(desc.maneuver);
+        section.setFloorLevelChange(desc.niveauDelta);
+        sections.push_back(std::move(section));
     }
 
     path.setSections(std::move(sections));
     return path;
 }
+
+QHash<QString, QString> EfaXmlParser::parseGenericAttributeList(ScopedXmlStreamReader &&reader) const
+{
+    return parseKeyValueList(std::move(reader), QLatin1String("genAttrElem"), QLatin1String("name"), QLatin1String("value"));
+}
