@@ -59,10 +59,10 @@ bool OpenTripPlannerGraphQLBackend::queryLocation(const LocationRequest &req, Lo
         gqlReq.setVariable(QStringLiteral("maxResults"), req.maximumResults());
         QJsonArray placeTypeFilter;
         if (req.types() & Location::Stop) {
-            placeTypeFilter.push_back(QStringLiteral("STOP"));
+            placeTypeFilter.push_back(m_apiVersion == QLatin1String("entur") ? QStringLiteral("stopPlace") : QStringLiteral("STOP"));
         }
         if (req.types() & (Location::RentedVehicleStation | Location::RentedVehicle)) {
-            placeTypeFilter.push_back(QStringLiteral("BICYCLE_RENT"));
+            placeTypeFilter.push_back(m_apiVersion == QLatin1String("entur") ? QStringLiteral("bicycleRent") : QStringLiteral("BICYCLE_RENT"));
         }
         // TODO: also supports BIKE_PARK, CAR_PARK
         gqlReq.setVariable(QStringLiteral("placeType"), placeTypeFilter);
@@ -134,6 +134,61 @@ bool OpenTripPlannerGraphQLBackend::queryStopover(const StopoverRequest &req, St
     return true;
 }
 
+static QString modeName(IndividualTransport::Mode mode)
+{
+    switch (mode) {
+        case IndividualTransport::Walk: return QStringLiteral("WALK");
+        case IndividualTransport::Bike: return QStringLiteral("BICYCLE");
+        case IndividualTransport::Car: return QStringLiteral("CAR");
+    }
+    return {};
+}
+
+static QString qualifierName(IndividualTransport::Qualifier qualifier)
+{
+    switch (qualifier) {
+        case IndividualTransport::None: return {};
+        case IndividualTransport::Park: return QStringLiteral("PARK");
+        case IndividualTransport::Rent: return QStringLiteral("RENT");
+        case IndividualTransport::Dropoff: return QStringLiteral("DROPOFF");
+        case IndividualTransport::Pickup: return QStringLiteral("PICKUP");
+    }
+    return {};
+}
+
+static void addEnturModes(QStringList &modes, const std::vector<IndividualTransport> &its)
+{
+    for (const auto &it : its) {
+        switch (it.mode()) {
+            case IndividualTransport::Bike:
+                // TODO park/rent variants only supported by Entur v3
+                modes.push_back(QStringLiteral("bicycle"));
+                break;
+            case IndividualTransport::Car:
+                switch (it.qualifier()) {
+                    case IndividualTransport::None:
+                        modes.push_back(QStringLiteral("car"));
+                        break;
+                    case IndividualTransport::Park:
+                        modes.push_back(QStringLiteral("car_park"));
+                        break;
+                    case IndividualTransport::Pickup:
+                        modes.push_back(QStringLiteral("car_pickup"));
+                        break;
+                    case IndividualTransport::Dropoff:
+                        modes.push_back(QStringLiteral("car_dropoff"));
+                        break;
+                    case IndividualTransport::Rent: // not supported
+                        break;
+                }
+                break;
+            case IndividualTransport::Walk:
+                modes.push_back(QStringLiteral("foot"));
+                break;
+        }
+    }
+}
+
 bool OpenTripPlannerGraphQLBackend::queryJourney(const JourneyRequest &req, JourneyReply *reply, QNetworkAccessManager *nam) const
 {
     if (!req.from().hasCoordinate() || !req.to().hasCoordinate()) {
@@ -148,7 +203,7 @@ bool OpenTripPlannerGraphQLBackend::queryJourney(const JourneyRequest &req, Jour
     gqlReq.setVariable(QStringLiteral("toLon"), req.to().longitude());
 
     auto dt = req.dateTime();
-    const auto context = requestContext(req).value<OpenTripPlannerRequestContext>();
+    const auto context = requestContextData(req).value<OpenTripPlannerRequestContext>();
     if (context.dateTime.isValid()) {
         dt = context.dateTime;
     }
@@ -162,28 +217,70 @@ bool OpenTripPlannerGraphQLBackend::queryJourney(const JourneyRequest &req, Jour
     gqlReq.setVariable(QStringLiteral("arriveBy"), req.dateTimeMode() == JourneyRequest::Arrival);
     gqlReq.setVariable(QStringLiteral("maxResults"), req.maximumResults());
     gqlReq.setVariable(QStringLiteral("lang"), preferredLanguage());
+    gqlReq.setVariable(QStringLiteral("withIntermediateStops"), req.includeIntermediateStops());
+    gqlReq.setVariable(QStringLiteral("withPaths"), req.includePaths());
     // TODO set context.searchWindow?
 
-    QJsonArray modes;
-    QJsonObject walkMode;
-    walkMode.insert(QStringLiteral("mode"), QStringLiteral("WALK"));
-    modes.push_back(walkMode);
-    if (req.modes() & JourneySection::PublicTransport) {
-        for (const auto &mode : m_supportedTransitModes) {
-            QJsonObject transitMode;
-            transitMode.insert(QStringLiteral("mode"), mode);
-            modes.push_back(transitMode);
+    if (m_apiVersion == QLatin1String("entur")) {
+        gqlReq.setVariable(QStringLiteral("allowBikeRental"), (req.modes() & JourneySection::RentedVehicle) != 0);
+        QStringList modes;
+        modes.push_back(QStringLiteral("foot"));
+        if (req.modes() & JourneySection::PublicTransport) {
+            modes.push_back(QStringLiteral("transit"));
         }
-    }
-    if (req.modes() & JourneySection::RentedVehicle) {
-        for (const auto &mode : m_supportedRentalModes) {
-            QJsonObject rentMode;
-            rentMode.insert(QStringLiteral("mode"), mode);
-            rentMode.insert(QStringLiteral("qualifier"), QStringLiteral("RENT"));
-            modes.push_back(rentMode);
+        if (req.modes() & JourneySection::RentedVehicle) {
+            modes.push_back(QStringLiteral("bicycle"));
         }
+        addEnturModes(modes, req.accessModes());
+        addEnturModes(modes, req.egressModes());
+
+        modes.removeDuplicates();
+        QJsonArray modesArray;
+        std::copy(modes.begin(), modes.end(), std::back_inserter(modesArray));
+        gqlReq.setVariable(QStringLiteral("modes"), modesArray);
+    } else {
+        struct Mode {
+            QString mode;
+            QString qualifier;
+        };
+        std::vector<Mode> modes;
+
+        if (req.modes() & JourneySection::PublicTransport) {
+            for (const auto &mode : m_supportedTransitModes) {
+                modes.push_back({ mode, {} });
+            }
+        }
+        if (req.modes() & JourneySection::RentedVehicle) {
+            for (const auto &mode : m_supportedRentalModes) {
+                modes.push_back({ mode, QStringLiteral("RENT") });
+            }
+        }
+        for (const auto &it : req.accessModes()) {
+            modes.push_back({ modeName(it.mode()), qualifierName(it.qualifier()) });
+        }
+        const auto modeLessThan = [](const Mode &lhs, const Mode &rhs) {
+            if (lhs.mode == rhs.mode) {
+                return lhs.qualifier < rhs.qualifier;
+            }
+            return lhs.mode < rhs.mode;
+        };
+        const auto modeEqual = [](const Mode &lhs, const Mode &rhs) {
+            return lhs.mode == rhs.mode && lhs.qualifier == rhs.qualifier;
+        };
+        std::sort(modes.begin(), modes.end(), modeLessThan);
+        modes.erase(std::unique(modes.begin(), modes.end(), modeEqual), modes.end());
+
+        QJsonArray modesArray;
+        for (const auto &mode : modes) {
+            QJsonObject modeObj;
+            modeObj.insert(QLatin1String("mode"), mode.mode);
+            if (!mode.qualifier.isEmpty()) {
+                modeObj.insert(QLatin1String("qualifier"), mode.qualifier);
+            }
+            modesArray.push_back(modeObj);
+        }
+        gqlReq.setVariable(QStringLiteral("modes"), modesArray);
     }
-    gqlReq.setVariable(QStringLiteral("modes"), modes);
 
     if (isLoggingEnabled()) {
         logRequest(req, gqlReq.networkRequest(), gqlReq.rawData());
